import numpy as np

delta = 1.0

def lnqp(q, p, length):
    """ apply the Liouville operator to (q, p), i.e. (iL)^n q and (iL)^n p """

    lnq = np.zeros(length, dtype=complex)
    lnp = np.zeros(length, dtype=complex)
    A = np.zeros(length, dtype=complex)
    B = np.zeros(length, dtype=complex)    
    A[1] = 2.*(q**3 - q)
    A[2] = 2.*(3*q**2 - 1)
    A[3] = 12.*q
    A[4] = 12.
    B[1] = p
    B[2] = 1

    lnq[0] = q
    lnp[0] = p
    lnq[1] =  B[1]
    lnp[1] = -A[1]
    lnq[2] = -A[1]*B[2]
    lnp[2] = -B[1]*A[2]
    lnq[3] =  A[1]**2*B[3] - A[2]*B[1]*B[2]
    lnp[3] = -B[1]**2*A[3] + B[2]*A[1]*A[2]
    lnq[4] = (3*A[1]*A[2]*B[1]*B[3] - A[3]*B[1]**2*B[2] 
              - A[1]**3*B[4] + A[1]*A[2]*B[2]**2)
    lnp[4] = (3*B[1]*B[2]*A[1]*A[3] - B[3]*A[1]**2*A[2] 
              - B[1]**3*A[4] + B[1]*B[2]*A[2]**2)
    lnq[5] = ((3*B[1]**2*B[3] + B[1]*B[2]**2)*(A[2]**2 + A[1]*A[3])
              - A[4]*B[1]**3*B[2] - 3*A[1]**2*A[2]*B[1]*B[4]
              - 5*A[1]**2*A[2]*B[2]*B[3] - 3*A[1]**2*A[2]*B[1]*B[4]
              + A[1]*A[3]*(2*B[1]*B[2]**2 + B[1]**2*B[3]) + A[1]**4*B[5])
    lnp[5] = (-(3*A[1]**2*A[3] + A[1]*A[2]**2)*(B[2]**2 + B[1]*B[3])
              + B[4]*A[1]**3*A[2] + 3*B[1]**2*B[2]*A[1]*A[4]
              + 5*B[1]**2*B[2]*A[2]*A[3] + 3*B[1]**2*B[2]*A[1]*A[4]
              - B[1]*B[3]*(2*A[1]*A[2]**2 + A[1]**2*A[3]) - B[1]**4*A[5])
    return (lnq, lnp)


def force(q):
    return -2.*(q**3 - q)

def energy(q, p):
    return 0.5*p*p + 0.5*q*q*q*q - q*q

def solution(q0, p0, time):
    pass
